@@ -1,13 +1,13 @@
 /* infokey.c -- compile ~/.infokey to ~/.info.
-   $Id: infokey.c,v 1.9.2.2 2007/06/27 17:10:58 karl Exp $
+   $Id: infokey.c,v 1.15 2007/07/01 21:20:30 karl Exp $
 
-   Copyright (C) 1999, 2001, 2002, 2003, 2004, 2007 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2003, 2004, 2005, 2007
+   Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,8 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
    Written by Andrew Bettison <andrewb@zip.com.au>. */
 
@@ -26,7 +25,7 @@
 #include "key.h"
 #include "getopt.h"
 
-static char *program_name = "infokey";
+char *program_name = "infokey";
 
 /* Non-zero means print version info only. */
 static int print_version_p = 0;
