@@ -1,13 +1,13 @@
 /* infomap.c -- keymaps for Info.
-   $Id: infomap.c,v 1.3 2004/03/14 00:57:29 karl Exp $
+   $Id: infomap.c,v 1.13 2007/07/01 21:20:30 karl Exp $
 
-   Copyright (C) 1993, 1997, 1998, 1999, 2001, 2002, 2003, 2004 Free Software
-   Foundation, Inc.
+   Copyright (C) 1993, 1997, 1998, 1999, 2001, 2002, 2003, 2004, 2007
+   Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,8 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
    Written by Brian Fox (bfox@ai.mit.edu). */
 
@@ -704,8 +703,8 @@ initialize_vi_like_keymaps ()
   map['H'].function = info_get_help_window;
   map['i'].function = info_index_search;
   map['I'].function = info_goto_invocation_node;
-  map['j'].function = info_down_line;
-  map['k'].function = info_up_line;
+  map['j'].function = info_next_line;
+  map['k'].function = info_prev_line;
   map['l'].function = info_history_node;
   map['m'].function = info_menu_item;
   map['n'].function = info_search_next;
@@ -1182,8 +1181,8 @@ static unsigned char default_vi_like_info_keys[] =
 	'H', NUL,			A_info_get_help_window,
 	'i', NUL,			A_info_index_search,
 	'I', NUL,			A_info_goto_invocation_node,
-	'j', NUL,			A_info_down_line,
-	'k', NUL,			A_info_up_line,
+	'j', NUL,			A_info_next_line,
+	'k', NUL,			A_info_prev_line,
 	'l', NUL,			A_info_history_node,
 	'm', NUL,			A_info_menu_item,
 	'n', NUL,			A_info_search_next,
