@@ -1,13 +1,13 @@
 /* index.c -- indexing for Texinfo.
-   $Id: index.c,v 1.17.2.1 2007/06/27 17:11:01 karl Exp $
+   $Id: index.c,v 1.24 2007/07/01 21:20:32 karl Exp $
 
-   Copyright (C) 1998, 1999, 2002, 2003, 2004, 2007 Free Software Foundation,
-   Inc.
+   Copyright (C) 1998, 1999, 2002, 2003, 2004, 2007
+   Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,10 +15,10 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "system.h"
+#include "mbswidth.h"
 #include "files.h"
 #include "footnote.h"
 #include "html.h"
@@ -657,7 +657,7 @@ sort_index (INDEX_ELT *index)
 static void
 insert_index_output_line_no (int line_number, int output_line_number_len)
 {
-  int last_column;
+  int last_column, out_line_no_width;
   int str_size = output_line_number_len + strlen (_("(line )"))
     + sizeof (NULL);
   char *out_line_no_str = (char *) xmalloc (str_size + 1);
@@ -673,16 +673,18 @@ insert_index_output_line_no (int line_number, int output_line_number_len)
     int i = output_paragraph_offset; 
     while (0 < i && output_paragraph[i-1] != '\n')
       i--;
-    last_column = output_paragraph_offset - i;
+    last_column = mbsnwidth ((char *)(output_paragraph + i),
+			     output_paragraph_offset - i, 0);
   }
 
-  if (last_column + strlen (out_line_no_str) > fill_column)
+  out_line_no_width = mbswidth (out_line_no_str, 0);
+  if (last_column + out_line_no_width > fill_column)
     {
       insert ('\n');
       last_column = 0;
     }
 
-  while (last_column + strlen (out_line_no_str) < fill_column)
+  while (last_column + out_line_no_width < fill_column)
     {
       insert (' ');
       last_column++;
@@ -711,10 +713,11 @@ cm_printindex (void)
   if (!handling_delayed_writes)
     line_number--;
 
-  if (xml && !docbook)
+  if (xml)
     {
       xml_insert_element (PRINTINDEX, START);
-      insert_string (index_name);
+      if (! docbook)
+	insert_string (index_name);
       xml_insert_element (PRINTINDEX, END);
     }
   else if (!handling_delayed_writes)
@@ -723,9 +726,6 @@ cm_printindex (void)
       char *index_command = xmalloc (command_len + 1);
 
       close_paragraph ();
-      if (docbook)
-        xml_begin_index ();
-
       sprintf (index_command, "@%s %s", command, index_name);
       register_delayed_write (index_command);
       free (index_command);
@@ -869,15 +869,14 @@ cm_printindex (void)
             }
           else if (xml && docbook)
             {
-              /* In the DocBook case, the expanded index entry is not
-                 good for us, since it was expanded for non-DocBook mode
-                 inside sort_index.  So we send the original entry text
-                 to be used with execute_string.  */
-              xml_insert_indexentry (index->entry_text, index_node);
+	      /* Let DocBook processor generate the index. */
             }
           else
             {
-              unsigned new_length = strlen (index->entry);
+#define MIN_ENTRY_COLUMNS 37
+	      /* Make sure there is enough space even if index->entry has zero
+		 width. */
+              unsigned new_length = strlen (index->entry) + MIN_ENTRY_COLUMNS;
 
               if (new_length < 50) /* minimum length used below */
                 new_length = 50;
@@ -894,7 +893,13 @@ cm_printindex (void)
                  @@ has turned into @. */
               if (!no_headers)
                 {
-                  sprintf (line, "* %-37s  ", index->entry);
+		  int width;
+
+		  width = mbswidth (index->entry, 0);
+                  sprintf (line, "* %*s  ", width < MIN_ENTRY_COLUMNS
+			   ? -(strlen (index->entry)
+			       + (MIN_ENTRY_COLUMNS - width))
+			   : 0, index->entry);
                   line[2 + strlen (index->entry)] = ':';
                   insert_string (line);
                   /* Make sure any non-macros in the node name are expanded.  */
