@@ -1,12 +1,12 @@
 /* install-info -- create Info directory entry(ies) for an Info file.
-   $Id: install-info.c,v 1.12.2.1 2007/06/27 17:02:28 karl Exp $
+   $Id: install-info.c,v 1.18 2007/07/01 21:20:33 karl Exp $
 
    Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2007 Free Software Foundation, Inc.
+   2005, 2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -15,11 +15,11 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*/
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "system.h"
 #include <getopt.h>
+#include <regex.h>
 
 static char *progname = "install-info";
 
@@ -119,6 +119,7 @@ struct menu_section
 
 struct option longopts[] =
 {
+  { "debug",     no_argument, NULL, 'g' },
   { "delete",    no_argument, NULL, 'r' },
   { "dir-file",  required_argument, NULL, 'd' },
   { "entry",     required_argument, NULL, 'e' },
@@ -129,10 +130,25 @@ struct option longopts[] =
   { "item",      required_argument, NULL, 'e' },
   { "quiet",     no_argument, NULL, 'q' },
   { "remove",    no_argument, NULL, 'r' },
-  { "section",   required_argument, NULL, 's' },
+  { "remove-exactly",    no_argument, NULL, 'x' },
+  { "section",           required_argument, NULL, 's' },
+  { "section-regex",     required_argument, NULL, 'R' },
+  { "silent",    no_argument, NULL, 'q' },
+  { "test",      no_argument, NULL, 'n' },
   { "version",   no_argument, NULL, 'V' },
   { 0 }
 };
+
+regex_t *psecreg = NULL;
+
+/* Nonzero means that the name specified for the info file will be used
+   (without removing .gz, .info extension or leading path) to match the
+   entries that must be removed.  */
+int remove_exactly = 0;
+
+/* Nonzero means --test was specified, to inhibit updating the dir file.  */
+int chicken_flag = 0;
+
 
 /* Error message functions.  */
 
@@ -383,6 +399,7 @@ menu_item_equal (const char *item, char term_char, const char *name)
      filename from the new dir entries, not the filename on the command
      line.  Not worrying about those things right now, though.  --karl,
      26mar04.  */
+  if (!remove_exactly) {
   while (*item_basename && !IS_SLASH (*item_basename)
 	 && *item_basename != term_char)
     item_basename++;
@@ -390,6 +407,7 @@ menu_item_equal (const char *item, char term_char, const char *name)
     item_basename = item;  /* no /, use original */
   else
     item_basename++;       /* have /, move past it */
+  }
     
   /* First, ITEM must actually match NAME (usually it won't).  */
   ret = strncasecmp (item_basename, name, name_len) == 0;
@@ -433,10 +451,10 @@ print_help (void)
 {
   printf (_("Usage: %s [OPTION]... [INFO-FILE [DIR-FILE]]\n\
 \n\
-Install or delete dir entries from INFO-FILE in the Info directory file\n\
-DIR-FILE.\n\
+Add or remove entries in INFO-FILE from the Info directory DIR-FILE.\n\
 \n\
 Options:\n\
+ --debug           report what is being done.\n\
  --delete          delete existing entries for INFO-FILE from DIR-FILE;\n\
                      don't insert any new entries.\n\
  --dir-file=NAME   specify file name of Info directory file.\n\
@@ -455,11 +473,18 @@ Options:\n\
                      An Info directory entry is actually a menu item.\n\
  --quiet           suppress warnings.\n\
  --remove          same as --delete.\n\
+ --remove-exactly  only remove if the info file name matches exactly;\n\
+                     .info and/or .gz suffixes are not ignored.\n\
  --section=SEC     put this file's entries in section SEC of the directory.\n\
                      If you specify more than one section, all the entries\n\
                      are added in each of the sections.\n\
                      If you don't specify any sections, they are determined\n\
                      from information in the Info file itself.\n\
+ --section-regex=R if an entry is added to a section that does not exist,\n\
+                     look for a section matching basic regular expression R\n\
+                     (and ignoring case) before starting a new section.\n\
+ --test            suppress updating of DIR-FILE.\n\
+ --silent          suppress warnings.\n\
  --version         display version information and exit.\n\
 "), progname);
 
@@ -470,13 +495,18 @@ Texinfo home page: http://www.gnu.org/software/texinfo/"));
 }
 
 
-/* If DIRFILE does not exist, create a minimal one (or abort).  If it
-   already exists, do nothing.  */
+/* If DIRFILE does not exist, and we are not in test mode, create a
+   minimal one (or abort).  If it already exists, do nothing.  */
 
-void
+static void
 ensure_dirfile_exists (char *dirfile)
 {
-  int desc = open (dirfile, O_RDONLY);
+  int desc;
+  
+  if (chicken_flag)
+    return;
+    
+  desc = open (dirfile, O_RDONLY);
   if (desc < 0 && errno == ENOENT)
     {
       FILE *f;
@@ -1154,9 +1184,13 @@ main (int argc, char **argv)
   /* Nonzero means --delete was specified (just delete existing entries).  */
   int delete_flag = 0;
   int something_deleted = 0;
-  /* Nonzero means -q was specified.  */
+
+  /* Nonzero means -quiet/--silent was specified.  */
   int quiet_flag = 0;
 
+  /* Nonzero means --debug was specified.  */
+  int debug_flag = 0;
+
   int i;
 
 #ifdef HAVE_SETLOCALE
@@ -1225,6 +1259,10 @@ main (int argc, char **argv)
           }
           break;
 
+        case 'g':
+          debug_flag = 1;
+          break;
+
         case 'h':
         case 'H':
           print_help ();
@@ -1240,6 +1278,10 @@ main (int argc, char **argv)
           infile = optarg;
           break;
 
+        case 'n':
+          chicken_flag = 1;
+          break;
+
         case 'q':
           quiet_flag = 1;
           break;
@@ -1248,6 +1290,27 @@ main (int argc, char **argv)
           delete_flag = 1;
           break;
 
+        case 'R':
+          {
+            int error;
+            if (psecreg)
+              warning (_("Extra regular expression specified, ignoring `%s'"),
+                       optarg, 0);
+            else
+              psecreg = (regex_t *) xmalloc (sizeof (regex_t));
+
+	    error = regcomp (psecreg, optarg, REG_ICASE|REG_NOSUB);
+            if (error != 0)
+              {
+                int errbuf_size = regerror (error, psecreg, NULL, 0);
+                char *errbuf = (char *) xmalloc (errbuf_size);
+                regerror (error, psecreg, errbuf, errbuf_size);
+                fatal (_("Error in regular expression `%s': %s"),
+                       optarg, errbuf);
+              };
+          }
+          break;
+
         case 's':
           {
             struct spec_section *next
@@ -1262,13 +1325,18 @@ main (int argc, char **argv)
         case 'V':
           printf ("install-info (GNU %s) %s\n", PACKAGE, VERSION);
           puts ("");
-      printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
+          printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n"),
 	      "2007");
           xexit (0);
 
+        case 'x':
+          delete_flag = 1;
+          remove_exactly = 1;
+          break;
+
         default:
           suggest_asking_for_help ();
         }
@@ -1294,6 +1362,8 @@ There is NO WARRANTY, to the extent permitted by law.\n"),
   /* Read the Info file and parse it into lines, unless we're deleting.  */
   if (!delete_flag)
     {
+      if (debug_flag)
+        printf ("debug: reading input file %s\n", infile);
       input_data = readfile (infile, &input_size, NULL, NULL, NULL);
       input_lines = findlines (input_data, input_size, &input_nlines);
     }
@@ -1343,6 +1413,8 @@ There is NO WARRANTY, to the extent permitted by law.\n"),
     }
 
   /* Now read in the Info dir file.  */
+  if (debug_flag)
+    printf ("debug: reading dir file %s\n", dirfile);
   dir_data = readfile (dirfile, &dir_size, ensure_dirfile_exists,
                        &opened_dirfilename, &compression_program);
   dir_lines = findlines (dir_data, dir_size, &dir_nlines);
@@ -1350,7 +1422,7 @@ There is NO WARRANTY, to the extent permitted by law.\n"),
   /* We will be comparing the entries in the dir file against the
      current filename, so need to strip off any directory prefix and/or
      [.info][.gz] suffix.  */
-  {
+  if (!remove_exactly) {
     char *infile_basename = infile + strlen (infile);
 
     if (HAVE_DRIVE (infile))
@@ -1360,7 +1432,8 @@ There is NO WARRANTY, to the extent permitted by law.\n"),
       infile_basename--;
 
     infile_sans_info = strip_info_suffix (infile_basename);
-  }
+  } else
+    infile_sans_info = xstrdup(infile);
 
   something_deleted
     = parse_dir_file (dir_lines, dir_nlines, &dir_nodes, infile_sans_info);
@@ -1375,6 +1448,31 @@ There is NO WARRANTY, to the extent permitted by law.\n"),
       struct menu_section *section;
       struct spec_section *spec;
 
+      for (spec = input_sections; spec; spec = spec->next)
+        {
+          int found = 0;
+          /* Check if the section specified (e.g. in the info file) exists */
+          for (node = dir_nodes; node && found == 0; node = node->next)
+            for (section = node->sections;
+                 section && found == 0;
+                 section = section->next)
+              if (strcmp (spec->name, section->name) == 0)
+                found = 1;
+
+          /* If it does not exist, but the user specified a regular expression,
+             try to find a section that matches this regex.  */
+          if (!found && psecreg)
+            for (node = dir_nodes; node && found == 0; node = node->next)
+              for (section = node->sections;
+                   section && found == 0;
+                   section = section->next)
+                if (regexec (psecreg, section->name, 0, NULL, 0) == 0)
+                  {
+                    spec->name = section->name;
+                    spec->missing = 0;
+                    found = 1;
+                  }
+        }
       for (node = dir_nodes; node; node = node->next)
         for (section = node->sections; section; section = section->next)
           {
@@ -1449,8 +1547,14 @@ There is NO WARRANTY, to the extent permitted by law.\n"),
   if (delete_flag && !something_deleted && !quiet_flag)
     warning (_("no entries found for `%s'; nothing deleted"), infile, 0);
 
-  output_dirfile (opened_dirfilename, dir_nlines, dir_lines, n_entries_to_add,
-                  entries_to_add, input_sections, compression_program);
+  if (debug_flag)
+    printf ("debug: writing dir file %s\n", opened_dirfilename);
+  if (chicken_flag)
+    printf ("test mode, not updating dir file %s\n", opened_dirfilename);
+  else
+    output_dirfile (opened_dirfilename, dir_nlines, dir_lines,
+                    n_entries_to_add, entries_to_add,
+                    input_sections, compression_program);
 
   xexit (0);
   return 0; /* Avoid bogus warnings.  */
